@@ -2,7 +2,7 @@
 """
 Classes for handling input/output prompts.
 
-$Id: Prompts.py 1850 2006-10-28 19:48:13Z fptest $"""
+$Id: Prompts.py 2855 2007-11-06 06:53:49Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez <fperez@colorado.edu>
@@ -30,6 +30,7 @@ from IPython.Itpl import ItplNS
 from IPython.ipstruct import Struct
 from IPython.macro import Macro
 from IPython.genutils import *
+from IPython.ipapi import TryNext
 
 #****************************************************************************
 #Color schemes for Prompts.
@@ -124,56 +125,59 @@ ROOT_SYMBOL    = "$#"[os.name=='nt' or os.getuid()==0]
 prompt_specials_color = {
     # Prompt/history count
     '%n' : '${self.col_num}' '${self.cache.prompt_count}' '${self.col_p}',
-    '\\#': '${self.col_num}' '${self.cache.prompt_count}' '${self.col_p}',
+    r'\#': '${self.col_num}' '${self.cache.prompt_count}' '${self.col_p}',
+    # Just the prompt counter number, WITHOUT any coloring wrappers, so users
+    # can get numbers displayed in whatever color they want.
+    r'\N': '${self.cache.prompt_count}',
     # Prompt/history count, with the actual digits replaced by dots.  Used
     # mainly in continuation prompts (prompt_in2)
-    '\\D': '${"."*len(str(self.cache.prompt_count))}',
+    r'\D': '${"."*len(str(self.cache.prompt_count))}',
     # Current working directory
-    '\\w': '${os.getcwd()}',
+    r'\w': '${os.getcwd()}',
     # Current time
-    '\\t' : '${time.strftime("%H:%M:%S")}',
+    r'\t' : '${time.strftime("%H:%M:%S")}',
     # Basename of current working directory.
     # (use os.sep to make this portable across OSes)
-    '\\W' : '${os.getcwd().split("%s")[-1]}' % os.sep,
+    r'\W' : '${os.getcwd().split("%s")[-1]}' % os.sep,
     # These X<N> are an extension to the normal bash prompts.  They return
     # N terms of the path, after replacing $HOME with '~'
-    '\\X0': '${os.getcwd().replace("%s","~")}' % HOME,
-    '\\X1': '${self.cwd_filt(1)}',
-    '\\X2': '${self.cwd_filt(2)}',
-    '\\X3': '${self.cwd_filt(3)}',
-    '\\X4': '${self.cwd_filt(4)}',
-    '\\X5': '${self.cwd_filt(5)}',
+    r'\X0': '${os.getcwd().replace("%s","~")}' % HOME,
+    r'\X1': '${self.cwd_filt(1)}',
+    r'\X2': '${self.cwd_filt(2)}',
+    r'\X3': '${self.cwd_filt(3)}',
+    r'\X4': '${self.cwd_filt(4)}',
+    r'\X5': '${self.cwd_filt(5)}',
     # Y<N> are similar to X<N>, but they show '~' if it's the directory
     # N+1 in the list.  Somewhat like %cN in tcsh.
-    '\\Y0': '${self.cwd_filt2(0)}',
-    '\\Y1': '${self.cwd_filt2(1)}',
-    '\\Y2': '${self.cwd_filt2(2)}',
-    '\\Y3': '${self.cwd_filt2(3)}',
-    '\\Y4': '${self.cwd_filt2(4)}',
-    '\\Y5': '${self.cwd_filt2(5)}',
+    r'\Y0': '${self.cwd_filt2(0)}',
+    r'\Y1': '${self.cwd_filt2(1)}',
+    r'\Y2': '${self.cwd_filt2(2)}',
+    r'\Y3': '${self.cwd_filt2(3)}',
+    r'\Y4': '${self.cwd_filt2(4)}',
+    r'\Y5': '${self.cwd_filt2(5)}',
     # Hostname up to first .
-    '\\h': HOSTNAME_SHORT,
+    r'\h': HOSTNAME_SHORT,
     # Full hostname
-    '\\H': HOSTNAME,
+    r'\H': HOSTNAME,
     # Username of current user
-    '\\u': USER,
+    r'\u': USER,
     # Escaped '\'
     '\\\\': '\\',
     # Newline
-    '\\n': '\n',
+    r'\n': '\n',
     # Carriage return
-    '\\r': '\r',
+    r'\r': '\r',
     # Release version
-    '\\v': __version__,
+    r'\v': __version__,
     # Root symbol ($ or #)
-    '\\$': ROOT_SYMBOL,
+    r'\$': ROOT_SYMBOL,
     }
 
 # A copy of the prompt_specials dictionary but with all color escapes removed,
 # so we can correctly compute the prompt length for the auto_rewrite method.
 prompt_specials_nocolor = prompt_specials_color.copy()
 prompt_specials_nocolor['%n'] = '${self.cache.prompt_count}'
-prompt_specials_nocolor['\\#'] = '${self.cache.prompt_count}'
+prompt_specials_nocolor[r'\#'] = '${self.cache.prompt_count}'
 
 # Add in all the InputTermColors color escapes as valid prompt characters.
 # They all get added as \\C_COLORNAME, so that we don't have any conflicts
@@ -183,7 +187,7 @@ prompt_specials_nocolor['\\#'] = '${self.cache.prompt_count}'
 input_colors = ColorANSI.InputTermColors
 for _color in dir(input_colors):
     if _color[0] != '_':
-        c_name = '\\C_'+_color
+        c_name = r'\C_'+_color
         prompt_specials_color[c_name] = getattr(input_colors,_color)
         prompt_specials_nocolor[c_name] = ''
 
@@ -211,8 +215,19 @@ def str_safe(arg):
         out = '<ERROR: %s>' % msg
     return out
 
-class BasePrompt:
+class BasePrompt(object):
     """Interactive prompt similar to Mathematica's."""
+
+    def _get_p_template(self):
+        return self._p_template
+
+    def _set_p_template(self,val):
+        self._p_template = val
+        self.set_p_str()
+
+    p_template = property(_get_p_template,_set_p_template,
+                          doc='Template for prompt string creation')
+
     def __init__(self,cache,sep,prompt,pad_left=False):
 
         # Hack: we access information about the primary prompt through the
@@ -228,7 +243,9 @@ class BasePrompt:
         # Flag to left-pad prompt strings to match the length of the primary
         # prompt
         self.pad_left = pad_left
-        # Set template to create each actual prompt (where numbers change)
+
+        # Set template to create each actual prompt (where numbers change).
+        # Use a property
         self.p_template = prompt
         self.set_p_str()
 
@@ -292,10 +309,15 @@ class BasePrompt:
         $HOME is always replaced with '~'.
         If depth==0, the full path is returned."""
 
-        cwd = os.getcwd().replace(HOME,"~").split(os.sep)
+        full_cwd = os.getcwd()
+        cwd = full_cwd.replace(HOME,"~").split(os.sep)
         if '~' in cwd and len(cwd) == depth+1:
             depth += 1
-        out = os.sep.join(cwd[-depth:])
+        drivepart = ''
+        if sys.platform == 'win32' and len(cwd) > depth:
+            drivepart = os.path.splitdrive(full_cwd)[0]
+        out = drivepart + '/'.join(cwd[-depth:])
+
         if out:
             return out
         else:
@@ -422,8 +444,12 @@ class CachedOutput:
 
         # Set input prompt strings and colors
         if cache_size == 0:
-            if ps1.find('%n') > -1 or ps1.find('\\#') > -1: ps1 = '>>> '
-            if ps2.find('%n') > -1 or ps2.find('\\#') > -1: ps2 = '... '
+            if ps1.find('%n') > -1 or ps1.find(r'\#') > -1 \
+                   or ps1.find(r'\N') > -1:
+                ps1 = '>>> '
+            if ps2.find('%n') > -1 or ps2.find(r'\#') > -1 \
+                   or ps2.find(r'\N') > -1:
+                ps2 = '... '
         self.ps1_str = self._set_prompt_str(ps1,'In [\\#]: ','>>> ')
         self.ps2_str = self._set_prompt_str(ps2,'   .\\D.: ','... ')
         self.ps_out_str = self._set_prompt_str(ps_out,'Out[\\#]: ','')
@@ -505,13 +531,6 @@ class CachedOutput:
             if self.do_full_cache:
                 cout_write(outprompt)
 
-            if isinstance(arg,Macro):
-                print 'Executing Macro...'
-                # in case the macro takes a long time to execute
-                Term.cout.flush()
-                self.shell.runlines(arg.value)
-                return None
-
             # and now call a possibly user-defined print mechanism
             manipulated_val = self.display(arg)
             
@@ -536,8 +555,10 @@ class CachedOutput:
         Do ip.set_hook("result_display", my_displayhook) for custom result
         display, e.g. when your own objects need special formatting.
         """
-
-        return self.shell.hooks.result_display(arg)
+        try:
+            return IPython.generics.result_display(arg)
+        except TryNext:            
+            return self.shell.hooks.result_display(arg)
 
     # Assign the default display method:
     display = _display
@@ -565,8 +586,8 @@ class CachedOutput:
         if self.do_full_cache:
             new_result = '_'+`self.prompt_count`
             to_main[new_result] = arg
-        self.user_ns.update(to_main)
-        self.user_ns['_oh'][self.prompt_count] = arg
+            self.user_ns.update(to_main)
+            self.user_ns['_oh'][self.prompt_count] = arg
 
     def flush(self):
         if not self.do_full_cache:
