@@ -4,7 +4,7 @@
 All the matplotlib support code was co-developed with John Hunter,
 matplotlib's author.
 
-$Id: Shell.py 1384 2006-06-29 20:04:37Z vivainio $"""
+$Id: Shell.py 2764 2007-09-15 14:16:21Z darren.dale $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez <fperez@colorado.edu>
@@ -18,29 +18,47 @@ __author__  = '%s <%s>' % Release.authors['Fernando']
 __license__ = Release.license
 
 # Code begins
+# Stdlib imports
 import __builtin__
 import __main__
 import Queue
+import inspect
 import os
-import signal
 import sys
+import thread
 import threading
 import time
 
+from signal import signal, SIGINT
+
+try:
+    import ctypes
+    HAS_CTYPES = True
+except ImportError:
+    HAS_CTYPES = False
+
+# IPython imports
 import IPython
-from IPython import ultraTB
-from IPython.genutils import Term,warn,error,flag_calls
+from IPython import ultraTB, ipapi
+from IPython.genutils import Term,warn,error,flag_calls, ask_yes_no
 from IPython.iplib import InteractiveShell
 from IPython.ipmaker import make_IPython
 from IPython.Magic import Magic
 from IPython.ipstruct import Struct
 
+# Globals
 # global flag to pass around information about Ctrl-C without exceptions
 KBINT = False
 
 # global flag to turn on/off Tk support.
 USE_TK = False
 
+# ID for the main thread, used for cross-thread exceptions
+MAIN_THREAD_ID = thread.get_ident()
+
+# Tag when runcode() is active, for exception handling
+CODE_RUN = None
+
 #-----------------------------------------------------------------------------
 # This class is trivial now, but I want to have it in to publish a clean
 # interface. Later when the internals are reorganized, code that uses this
@@ -61,6 +79,22 @@ class IPShell:
             sys.exit()
 
 #-----------------------------------------------------------------------------
+def kill_embedded(self,parameter_s=''):
+    """%kill_embedded : deactivate for good the current embedded IPython.
+
+    This function (after asking for confirmation) sets an internal flag so that
+    an embedded IPython will never activate again.  This is useful to
+    permanently disable a shell that is being called inside a loop: once you've
+    figured out what you needed from it, you may then kill it and the program
+    will then continue to run without the interactive shell interfering again.
+    """
+    
+    kill = ask_yes_no("Are you sure you want to kill this embedded instance "
+                     "(y/n)? [y/N] ",'n')
+    if kill:
+        self.shell.embedded_active = False
+        print "This embedded IPython will not reactivate anymore once you exit."
+    
 class IPShellEmbed:
     """Allow embedding an IPython shell into a running program.
 
@@ -135,6 +169,9 @@ class IPShellEmbed:
                                embedded=True,
                                user_ns=user_ns)
 
+        ip = ipapi.IPApi(self.IP)
+        ip.expose_magic("kill_embedded",kill_embedded)
+
         # copy our own displayhook also
         self.sys_displayhook_embed = sys.displayhook
         # and leave the system's display hook clean
@@ -178,6 +215,14 @@ class IPShellEmbed:
         The optional keyword parameter dummy controls whether the call
         actually does anything.  """
 
+        # If the user has turned it off, go away
+        if not self.IP.embedded_active:
+            return
+
+        # Normal exits from interactive mode set this flag, so the shell can't
+        # re-enter (it checks this variable at the start of interactive mode).
+        self.IP.exit_now = False
+
         # Allow the dummy parameter to override the global __dummy_mode
         if dummy or (dummy != 0 and self.__dummy_mode):
             return
@@ -185,7 +230,7 @@ class IPShellEmbed:
         # Set global subsystems (display,completions) to our values
         sys.displayhook = self.sys_displayhook_embed
         if self.IP.has_readline:
-            self.IP.readline.set_completer(self.IP.Completer.complete)
+            self.IP.set_completer()
 
         if self.banner and header:
             format = '%s\n%s\n'
@@ -241,20 +286,58 @@ class IPShellEmbed:
         self.exit_msg = exit_msg
 
 #-----------------------------------------------------------------------------
-def sigint_handler (signum,stack_frame):
-    """Sigint handler for threaded apps.
+if HAS_CTYPES:
+    #  Add async exception support.  Trick taken from:
+    # http://sebulba.wikispaces.com/recipe+thread2
+    def _async_raise(tid, exctype):
+        """raises the exception, performs cleanup if needed"""
+        if not inspect.isclass(exctype):
+            raise TypeError("Only types can be raised (not instances)")
+        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid,
+                                                         ctypes.py_object(exctype))
+        if res == 0:
+            raise ValueError("invalid thread id")
+        elif res != 1:
+            # """if it returns a number greater than one, you're in trouble, 
+            # and you should call it again with exc=NULL to revert the effect"""
+            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, 0)
+            raise SystemError("PyThreadState_SetAsyncExc failed")
 
-    This is a horrible hack to pass information about SIGINT _without_ using
-    exceptions, since I haven't been able to properly manage cross-thread
-    exceptions in GTK/WX.  In fact, I don't think it can be done (or at least
-    that's my understanding from a c.l.py thread where this was discussed)."""
+    def sigint_handler (signum,stack_frame):
+        """Sigint handler for threaded apps.
+
+        This is a horrible hack to pass information about SIGINT _without_
+        using exceptions, since I haven't been able to properly manage
+        cross-thread exceptions in GTK/WX.  In fact, I don't think it can be
+        done (or at least that's my understanding from a c.l.py thread where
+        this was discussed)."""
+
+        global KBINT
+
+        if CODE_RUN:
+            _async_raise(MAIN_THREAD_ID,KeyboardInterrupt)
+        else:
+            KBINT = True
+            print '\nKeyboardInterrupt - Press <Enter> to continue.',
+            Term.cout.flush()
+
+else:
+    def sigint_handler (signum,stack_frame):
+        """Sigint handler for threaded apps.
+
+        This is a horrible hack to pass information about SIGINT _without_
+        using exceptions, since I haven't been able to properly manage
+        cross-thread exceptions in GTK/WX.  In fact, I don't think it can be
+        done (or at least that's my understanding from a c.l.py thread where
+        this was discussed)."""
+
+        global KBINT
+
+        print '\nKeyboardInterrupt - Press <Enter> to continue.',
+        Term.cout.flush()
+        # Set global flag so that runsource can know that Ctrl-C was hit
+        KBINT = True
 
-    global KBINT
-    
-    print '\nKeyboardInterrupt - Press <Enter> to continue.',
-    Term.cout.flush()
-    # Set global flag so that runsource can know that Ctrl-C was hit
-    KBINT = True
 
 class MTInteractiveShell(InteractiveShell):
     """Simple multi-threaded shell."""
@@ -283,7 +366,7 @@ class MTInteractiveShell(InteractiveShell):
         # A queue to hold the code to be executed.  A scalar variable is NOT
         # enough, because uses like macros cause reentrancy.
         self.code_queue = Queue.Queue()
-        
+
         # Stuff to do at closing time
         self._kill = False
         on_kill = kw.get('on_kill')
@@ -338,17 +421,15 @@ class MTInteractiveShell(InteractiveShell):
 
         Multithreaded wrapper around IPython's runcode()."""
 
+        global CODE_RUN
+
+        # Exceptions need to be raised differently depending on which thread is
+        # active
+        CODE_RUN = True
+        
         # lock thread-protected stuff
         got_lock = self.thread_ready.acquire(False)
 
-        # Install sigint handler
-        try:
-            signal.signal(signal.SIGINT, sigint_handler)
-        except SystemError:
-            # This happens under Windows, which seems to have all sorts
-            # of problems with signal handling.  Oh well...
-            pass
-
         if self._kill:
             print >>Term.cout, 'Closing threads...',
             Term.cout.flush()
@@ -356,6 +437,15 @@ class MTInteractiveShell(InteractiveShell):
                 tokill()
             print >>Term.cout, 'Done.'
 
+        # Install sigint handler.  We do it every time to ensure that if user
+        # code modifies it, we restore our own handling.
+        try:
+            signal(SIGINT,sigint_handler)
+        except SystemError:
+            # This happens under Windows, which seems to have all sorts
+            # of problems with signal handling.  Oh well...
+            pass
+
         # Flush queue of pending code by calling the run methood of the parent
         # class with all items which may be in the queue.
         while 1:
@@ -372,6 +462,9 @@ class MTInteractiveShell(InteractiveShell):
         # We're done with thread-protected variables
         if got_lock:
             self.thread_ready.release()
+
+        # We're done...
+        CODE_RUN = False
         # This MUST return true for gtk threading to work
         return True
 
@@ -446,8 +539,7 @@ class MatplotlibShellBase:
         user_ns = IPython.ipapi.make_user_ns(user_ns)
 
         exec ("import matplotlib\n"
-              "import matplotlib.pylab as pylab\n"
-              "from matplotlib.pylab import *") in user_ns
+              "import matplotlib.pylab as pylab\n") in user_ns
         
         # Build matplotlib info banner
         b="""
@@ -555,8 +647,14 @@ def hijack_wx():
     """    
     def dummy_mainloop(*args, **kw):
         pass
-    import wxPython
-    ver = wxPython.__version__
+
+    try:
+        import wx
+    except ImportError:
+        # For very old versions of WX
+        import wxPython as wx
+        
+    ver = wx.__version__
     orig_mainloop = None
     if ver[:3] >= '2.5':
         import wx
@@ -566,8 +664,8 @@ def hijack_wx():
         orig_mainloop = core.PyApp_MainLoop
         core.PyApp_MainLoop = dummy_mainloop
     elif ver[:3] == '2.4':
-        orig_mainloop = wxPython.wxc.wxPyApp_MainLoop
-        wxPython.wxc.wxPyApp_MainLoop = dummy_mainloop
+        orig_mainloop = wx.wxc.wxPyApp_MainLoop
+        wx.wxc.wxPyApp_MainLoop = dummy_mainloop
     else:
         warn("Unable to find either wxPython version 2.4 or >= 2.5.")
     return orig_mainloop
@@ -586,13 +684,45 @@ def hijack_gtk():
     gtk.main = dummy_mainloop
     return orig_mainloop
 
+def hijack_qt():
+    """Modifies PyQt's mainloop with a dummy so user code does not 
+    block IPython.  This function returns the original 
+    `qt.qApp.exec_loop` function that has been hijacked.
+    """    
+    def dummy_mainloop(*args, **kw):
+        pass
+    import qt
+    orig_mainloop = qt.qApp.exec_loop
+    qt.qApp.exec_loop = dummy_mainloop
+    qt.QApplication.exec_loop = dummy_mainloop
+    return orig_mainloop
+
+def hijack_qt4():
+    """Modifies PyQt4's mainloop with a dummy so user code does not
+    block IPython.  This function returns the original 
+    `QtGui.qApp.exec_` function that has been hijacked.
+    """    
+    def dummy_mainloop(*args, **kw):
+        pass
+    from PyQt4 import QtGui, QtCore
+    orig_mainloop = QtGui.qApp.exec_
+    QtGui.qApp.exec_ = dummy_mainloop
+    QtGui.QApplication.exec_ = dummy_mainloop
+    QtCore.QCoreApplication.exec_ = dummy_mainloop
+    return orig_mainloop
+
 #-----------------------------------------------------------------------------
 # The IPShell* classes below are the ones meant to be run by external code as
 # IPython instances.  Note that unless a specific threading strategy is
 # desired, the factory function start() below should be used instead (it
 # selects the proper threaded class).
 
-class IPShellGTK(threading.Thread):
+class IPThread(threading.Thread):
+    def run(self):
+        self.IP.mainloop(self._banner)
+        self.IP.kill()
+
+class IPShellGTK(IPThread):
     """Run a gtk mainloop() in a separate thread.
     
     Python commands can be passed to the thread where they will be executed.
@@ -628,10 +758,6 @@ class IPShellGTK(threading.Thread):
 
         threading.Thread.__init__(self)
 
-    def run(self):
-        self.IP.mainloop(self._banner)
-        self.IP.kill()
-
     def mainloop(self,sys_exit=0,banner=None):
 
         self._banner = banner
@@ -645,7 +771,7 @@ class IPShellGTK(threading.Thread):
         if sys.platform != 'win32':
             try:
                 if self.gtk.gtk_version[0] >= 2:
-                    self.gtk.threads_init()
+                    self.gtk.gdk.threads_init()
             except AttributeError:
                 pass
             except RuntimeError:
@@ -658,10 +784,11 @@ class IPShellGTK(threading.Thread):
                       'Note that matplotlib will most likely not work in its '
                       'current state!')
                 self.IP.InteractiveTB()
+
         self.start()
-        self.gtk.threads_enter()
+        self.gtk.gdk.threads_enter()
         self.gtk_mainloop()
-        self.gtk.threads_leave()
+        self.gtk.gdk.threads_leave()
         self.join()
 
     def on_timer(self):
@@ -674,7 +801,8 @@ class IPShellGTK(threading.Thread):
         time.sleep(0.01)
         return True
 
-class IPShellWX(threading.Thread):
+
+class IPShellWX(IPThread):
     """Run a wx mainloop() in a separate thread.
     
     Python commands can be passed to the thread where they will be executed.
@@ -706,7 +834,7 @@ class IPShellWX(threading.Thread):
                     error('Requested wxPython version %s could not be loaded' %
                                                                wantedwxversion)
 
-        import wxPython.wx as wx
+        import wx
 
         threading.Thread.__init__(self)
         self.wx = wx
@@ -715,7 +843,6 @@ class IPShellWX(threading.Thread):
         # Allows us to use both Tk and GTK.
         self.tk = get_tk()
         
-        
         # HACK: slot for banner in self; it will be passed to the mainloop
         # method only and .run() needs it.  The actual value will be set by
         # .mainloop().
@@ -728,30 +855,26 @@ class IPShellWX(threading.Thread):
             self.app.agent.timer.Stop()
             self.app.ExitMainLoop()
 
-    def run(self):
-        self.IP.mainloop(self._banner)
-        self.IP.kill()
-
     def mainloop(self,sys_exit=0,banner=None):
 
         self._banner = banner
         
         self.start()
 
-        class TimerAgent(self.wx.wxMiniFrame):
+        class TimerAgent(self.wx.MiniFrame):
             wx = self.wx
             IP = self.IP
             tk = self.tk
             def __init__(self, parent, interval):
-                style = self.wx.wxDEFAULT_FRAME_STYLE | self.wx.wxTINY_CAPTION_HORIZ
-                self.wx.wxMiniFrame.__init__(self, parent, -1, ' ', pos=(200, 200),
+                style = self.wx.DEFAULT_FRAME_STYLE | self.wx.TINY_CAPTION_HORIZ
+                self.wx.MiniFrame.__init__(self, parent, -1, ' ', pos=(200, 200),
                                              size=(100, 100),style=style)
                 self.Show(False)
                 self.interval = interval
-                self.timerId = self.wx.wxNewId()                                
+                self.timerId = self.wx.NewId()                                
 
             def StartWork(self):
-                self.timer = self.wx.wxTimer(self, self.timerId)
+                self.timer = self.wx.Timer(self, self.timerId)
                 self.wx.EVT_TIMER(self,  self.timerId, self.OnTimer)
                 self.timer.Start(self.interval)
 
@@ -759,22 +882,22 @@ class IPShellWX(threading.Thread):
                 update_tk(self.tk)
                 self.IP.runcode()
 
-        class App(self.wx.wxApp):
+        class App(self.wx.App):
             wx = self.wx
             TIMEOUT = self.TIMEOUT
             def OnInit(self):
                 'Create the main window and insert the custom frame'
                 self.agent = TimerAgent(None, self.TIMEOUT)
-                self.agent.Show(self.wx.false)
+                self.agent.Show(False)
                 self.agent.StartWork()
-                return self.wx.true
-        
+                return True
+
         self.app = App(redirect=False)
         self.wx_mainloop(self.app)
         self.join()
 
 
-class IPShellQt(threading.Thread):
+class IPShellQt(IPThread):
     """Run a Qt event loop in a separate thread.
     
     Python commands can be passed to the thread where they will be executed.
@@ -783,30 +906,18 @@ class IPShellQt(threading.Thread):
     
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,user_global_ns=None,
-                 debug=0,shell_class=MTInteractiveShell):
-        
+    def __init__(self, argv=None, user_ns=None, user_global_ns=None,
+                 debug=0, shell_class=MTInteractiveShell):
+
         import qt
 
-        class newQApplication:
-            def __init__( self ):
-                self.QApplication = qt.QApplication
-                
-            def __call__( *args, **kwargs ):
-                return qt.qApp
-
-            def exec_loop( *args, **kwargs ):
-                pass
-
-            def __getattr__( self, name ):
-                return getattr( self.QApplication, name )
-          
-        qt.QApplication = newQApplication()
+        self.exec_loop = hijack_qt()
 
         # Allows us to use both Tk and QT.
         self.tk = get_tk()
 
-        self.IP = make_IPython(argv,user_ns=user_ns,
+        self.IP = make_IPython(argv,
+                               user_ns=user_ns,
                                user_global_ns=user_global_ns,
                                debug=debug,
                                shell_class=shell_class,
@@ -819,36 +930,35 @@ class IPShellQt(threading.Thread):
         
         threading.Thread.__init__(self)
 
-    def run(self):
-        self.IP.mainloop(self._banner)
-        self.IP.kill()
-
-    def mainloop(self,sys_exit=0,banner=None):
+    def mainloop(self, sys_exit=0, banner=None):
 
         import qt
 
         self._banner = banner
 
         if qt.QApplication.startingUp():
-          a = qt.QApplication.QApplication(sys.argv)
+            a = qt.QApplication(sys.argv)
+
         self.timer = qt.QTimer()
-        qt.QObject.connect( self.timer, qt.SIGNAL( 'timeout()' ), self.on_timer )
+        qt.QObject.connect(self.timer,
+                           qt.SIGNAL('timeout()'),
+                           self.on_timer)
 
         self.start()
-        self.timer.start( self.TIMEOUT, True )
+        self.timer.start(self.TIMEOUT, True)
         while True:
             if self.IP._kill: break
-            qt.qApp.exec_loop()
+            self.exec_loop()
         self.join()
 
     def on_timer(self):
         update_tk(self.tk)
         result = self.IP.runcode()
-        self.timer.start( self.TIMEOUT, True )
+        self.timer.start(self.TIMEOUT, True)
         return result
 
 
-class IPShellQt4(threading.Thread):
+class IPShellQt4(IPThread):
     """Run a Qt event loop in a separate thread.
 
     Python commands can be passed to the thread where they will be executed.
@@ -857,30 +967,23 @@ class IPShellQt4(threading.Thread):
 
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,user_global_ns=None,
-                 debug=0,shell_class=MTInteractiveShell):
+    def __init__(self, argv=None, user_ns=None, user_global_ns=None,
+                 debug=0, shell_class=MTInteractiveShell):
 
         from PyQt4 import QtCore, QtGui
 
-        class newQApplication:
-            def __init__( self ):
-                self.QApplication = QtGui.QApplication
+        if QtCore.PYQT_VERSION_STR == '4.3':
+            warn('''PyQt4 version 4.3 detected.
+If you experience repeated threading warnings, please update PyQt4.
+''')
 
-            def __call__( *args, **kwargs ):
-                return QtGui.qApp
-
-            def exec_loop( *args, **kwargs ):
-                pass
-
-            def __getattr__( self, name ):
-                return getattr( self.QApplication, name )
-
-        QtGui.QApplication = newQApplication()
+        self.exec_ = hijack_qt4()
 
         # Allows us to use both Tk and QT.
         self.tk = get_tk()
 
-        self.IP = make_IPython(argv,user_ns=user_ns,
+        self.IP = make_IPython(argv,
+                               user_ns=user_ns,
                                user_global_ns=user_global_ns,
                                debug=debug,
                                shell_class=shell_class,
@@ -893,37 +996,47 @@ class IPShellQt4(threading.Thread):
 
         threading.Thread.__init__(self)
 
-    def run(self):
-        self.IP.mainloop(self._banner)
-        self.IP.kill()
-
-    def mainloop(self,sys_exit=0,banner=None):
+    def mainloop(self, sys_exit=0, banner=None):
 
         from PyQt4 import QtCore, QtGui
 
         self._banner = banner
 
         if QtGui.QApplication.startingUp():
-          a = QtGui.QApplication.QApplication(sys.argv)
+            a = QtGui.QApplication(sys.argv)
+
         self.timer = QtCore.QTimer()
-        QtCore.QObject.connect( self.timer, QtCore.SIGNAL( 'timeout()' ), self.on_timer )
+        QtCore.QObject.connect(self.timer,
+                               QtCore.SIGNAL('timeout()'),
+                               self.on_timer)
 
         self.start()
-        self.timer.start( self.TIMEOUT )
+        self.timer.start(self.TIMEOUT)
         while True:
             if self.IP._kill: break
-            QtGui.qApp.exec_()
+            self.exec_()
         self.join()
 
     def on_timer(self):
         update_tk(self.tk)
         result = self.IP.runcode()
-        self.timer.start( self.TIMEOUT )
+        self.timer.start(self.TIMEOUT)
         return result
 
 
-# A set of matplotlib public IPython shell classes, for single-threaded
-# (Tk* and FLTK* backends) and multithreaded (GTK* and WX* backends) use.
+# A set of matplotlib public IPython shell classes, for single-threaded (Tk*
+# and FLTK*) and multithreaded (GTK*, WX* and Qt*) backends to use.
+def _load_pylab(user_ns):
+    """Allow users to disable pulling all of pylab into the top-level
+    namespace.
+
+    This little utility must be called AFTER the actual ipython instance is
+    running, since only then will the options file have been fully parsed."""
+    
+    ip = IPython.ipapi.get()
+    if ip.options.pylab_import_all:
+        exec "from matplotlib.pylab import *" in user_ns
+
 class IPShellMatplotlib(IPShell):
     """Subclass IPShell with MatplotlibShell as the internal shell.
 
@@ -934,6 +1047,7 @@ class IPShellMatplotlib(IPShell):
     def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
         IPShell.__init__(self,argv,user_ns,user_global_ns,debug,
                          shell_class=MatplotlibShell)
+        _load_pylab(self.IP.user_ns)
 
 class IPShellMatplotlibGTK(IPShellGTK):
     """Subclass IPShellGTK with MatplotlibMTShell as the internal shell.
@@ -943,6 +1057,7 @@ class IPShellMatplotlibGTK(IPShellGTK):
     def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
         IPShellGTK.__init__(self,argv,user_ns,user_global_ns,debug,
                             shell_class=MatplotlibMTShell)
+        _load_pylab(self.IP.user_ns)
 
 class IPShellMatplotlibWX(IPShellWX):
     """Subclass IPShellWX with MatplotlibMTShell as the internal shell.
@@ -952,6 +1067,7 @@ class IPShellMatplotlibWX(IPShellWX):
     def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
         IPShellWX.__init__(self,argv,user_ns,user_global_ns,debug,
                            shell_class=MatplotlibMTShell)
+        _load_pylab(self.IP.user_ns)
 
 class IPShellMatplotlibQt(IPShellQt):
     """Subclass IPShellQt with MatplotlibMTShell as the internal shell.
@@ -961,6 +1077,7 @@ class IPShellMatplotlibQt(IPShellQt):
     def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
         IPShellQt.__init__(self,argv,user_ns,user_global_ns,debug,
                            shell_class=MatplotlibMTShell)
+        _load_pylab(self.IP.user_ns)
 
 class IPShellMatplotlibQt4(IPShellQt4):
     """Subclass IPShellQt4 with MatplotlibMTShell as the internal shell.
@@ -970,6 +1087,7 @@ class IPShellMatplotlibQt4(IPShellQt4):
     def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
         IPShellQt4.__init__(self,argv,user_ns,user_global_ns,debug,
                            shell_class=MatplotlibMTShell)
+        _load_pylab(self.IP.user_ns)
 
 #-----------------------------------------------------------------------------
 # Factory functions to actually start the proper thread-aware shell
