@@ -5,7 +5,7 @@ General purpose utilities.
 This is a grab-bag of stuff I find useful in most programs I write. Some of
 these things are also convenient when working at the command line.
 
-$Id: genutils.py 1930 2006-11-26 17:22:13Z vivainio $"""
+$Id: genutils.py 2872 2007-11-25 17:58:05Z fperez $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
@@ -22,6 +22,7 @@ __license__ = Release.license
 # required modules from the Python standard library
 import __main__
 import commands
+import doctest
 import os
 import re
 import shlex
@@ -33,8 +34,10 @@ import types
 import warnings
 
 # Other IPython utilities
+import IPython
 from IPython.Itpl import Itpl,itpl,printpl
-from IPython import DPyGetOpt
+from IPython import DPyGetOpt, platutils
+from IPython.generics import result_display
 from path import path
 if os.name == "nt":
     from IPython.winconsole import get_console_size
@@ -67,6 +70,10 @@ class IOStream:
                 # if we get here, something is seriously broken.
                 print >> sys.stderr, \
                       'ERROR - failed to write data to stream:', self.stream
+
+    def close(self):
+        pass
+                
         
 class IOTerm:
     """ Term holds the file or file-like objects for handling I/O operations.
@@ -163,15 +170,33 @@ StringTypes = types.StringTypes
 # If possible (Unix), use the resource module instead of time.clock()
 try:
     import resource
+    def clocku():
+        """clocku() -> floating point number
+
+        Return the *USER* CPU time in seconds since the start of the process.
+        This is done via a call to resource.getrusage, so it avoids the
+        wraparound problems in time.clock()."""
+        
+        return resource.getrusage(resource.RUSAGE_SELF)[0]
+    
+    def clocks():
+        """clocks() -> floating point number
+
+        Return the *SYSTEM* CPU time in seconds since the start of the process.
+        This is done via a call to resource.getrusage, so it avoids the
+        wraparound problems in time.clock()."""
+        
+        return resource.getrusage(resource.RUSAGE_SELF)[1]
+    
     def clock():
         """clock() -> floating point number
 
-        Return the CPU time in seconds (user time only, system time is
-        ignored) since the start of the process.  This is done via a call to
-        resource.getrusage, so it avoids the wraparound problems in
-        time.clock()."""
-        
-        return resource.getrusage(resource.RUSAGE_SELF)[0]
+        Return the *TOTAL USER+SYSTEM* CPU time in seconds since the start of
+        the process.  This is done via a call to resource.getrusage, so it
+        avoids the wraparound problems in time.clock()."""
+
+        u,s = resource.getrusage(resource.RUSAGE_SELF)[:2] 
+        return u+s
     
     def clock2():
         """clock2() -> (t_user,t_system)
@@ -180,7 +205,9 @@ try:
         return resource.getrusage(resource.RUSAGE_SELF)[:2]
 
 except ImportError:
-    clock = time.clock
+    # There is no distinction of user/system time under windows, so we just use
+    # time.clock() for everything...
+    clocku = clocks = clock = time.clock
     def clock2():
         """Under windows, system CPU time can't be measured.
 
@@ -241,7 +268,14 @@ def arg_split(s,posix=False):
     This is a modified version of the standard library's shlex.split()
     function, but with a default of posix=False for splitting, so that quotes
     in inputs are respected."""
-    
+
+    # XXX - there may be unicode-related problems here!!!  I'm not sure that
+    # shlex is truly unicode-safe, so it might be necessary to do
+    #
+    # s = s.encode(sys.stdin.encoding)
+    #
+    # first, to ensure that shlex gets a normal string.  Input from anyone who
+    # knows more about unicode and shlex than I would be good to have here...
     lex = shlex.shlex(s, posix=posix)
     lex.whitespace_split = True
     return list(lex)
@@ -267,6 +301,19 @@ def system(cmd,verbose=0,debug=0,header=''):
     if not debug: stat = os.system(cmd)
     return stat
 
+def abbrev_cwd():
+    """ Return abbreviated version of cwd, e.g. d:mydir """
+    cwd = os.getcwd()
+    drivepart = ''
+    if sys.platform == 'win32':
+        if len(cwd) < 4:
+            return cwd
+        drivepart = os.path.splitdrive(cwd)[0]
+    return (drivepart + (
+        cwd == '/' and '/' or \
+        os.path.basename(cwd)))
+    
+
 # This function is used by ipython in a lot of places to make system calls.
 # We need it to be slightly different under win32, due to the vagaries of
 # 'network shares'.  A win32 override is below.
@@ -291,8 +338,11 @@ def shell(cmd,verbose=0,debug=0,header=''):
     if verbose or debug: print header+cmd
     # flush stdout so we don't mangle python's buffering
     sys.stdout.flush()
+    
     if not debug:
+        platutils.set_term_title("IPy " + cmd)
         os.system(cmd)
+        platutils.set_term_title("IPy " + abbrev_cwd())
 
 # override shell() for win32 to deal with network shares
 if os.name in ('nt','dos'):
@@ -570,9 +620,9 @@ def process_cmdline(argv,names=[],defaults={},usage=''):
 
     try:
         getopt.processArguments(argv)
-    except:
+    except DPyGetOpt.ArgumentError, exc:
         print usage
-        warn(`sys.exc_value`,level=4)
+        warn('"%s"' % exc,level=4)
 
     defaults.update(getopt.optionValues)
     args = getopt.freeValues
@@ -762,6 +812,60 @@ def flag_calls(func):
     wrapper.__doc__ = func.__doc__
     return wrapper
 
+#----------------------------------------------------------------------------
+def dhook_wrap(func,*a,**k):
+    """Wrap a function call in a sys.displayhook controller.
+
+    Returns a wrapper around func which calls func, with all its arguments and
+    keywords unmodified, using the default sys.displayhook.  Since IPython
+    modifies sys.displayhook, it breaks the behavior of certain systems that
+    rely on the default behavior, notably doctest.
+    """
+
+    def f(*a,**k):
+    
+        dhook_s = sys.displayhook
+        sys.displayhook = sys.__displayhook__
+        try:
+            out = func(*a,**k)
+        finally:
+            sys.displayhook = dhook_s
+
+        return out
+
+    f.__doc__ = func.__doc__
+    return f
+
+#----------------------------------------------------------------------------
+def doctest_reload():
+    """Properly reload doctest to reuse it interactively.
+
+    This routine:
+
+      - reloads doctest
+      
+      - resets its global 'master' attribute to None, so that multiple uses of
+      the module interactively don't produce cumulative reports.
+    
+      - Monkeypatches its core test runner method to protect it from IPython's
+      modified displayhook.  Doctest expects the default displayhook behavior
+      deep down, so our modification breaks it completely.  For this reason, a
+      hard monkeypatch seems like a reasonable solution rather than asking
+      users to manually use a different doctest runner when under IPython."""
+
+    import doctest
+    reload(doctest)
+    doctest.master=None
+
+    try:
+        doctest.DocTestRunner
+    except AttributeError:
+        # This is only for python 2.3 compatibility, remove once we move to
+        # 2.4 only.
+        pass
+    else:
+        doctest.DocTestRunner.run = dhook_wrap(doctest.DocTestRunner.run)
+
 #----------------------------------------------------------------------------
 class HomeDirError(Error):
     pass
@@ -776,6 +880,16 @@ def get_home_dir():
 
     isdir = os.path.isdir
     env = os.environ
+    
+    # first, check py2exe distribution root directory for _ipython.
+    # This overrides all. Normally does not exist.
+    
+    if '\\library.zip\\' in IPython.__file__.lower():
+        root, rest = IPython.__file__.lower().split('library.zip')
+        if isdir(root + '_ipython'):
+            os.environ["IPYKITROOT"] = root.rstrip('\\')
+            return root
+    
     try:
         homedir = env['HOME']
         if not isdir(homedir):
@@ -832,6 +946,7 @@ class LSString(str):
         .l (or .list) : value as list (split on newlines).
         .n (or .nlstr): original value (the string itself).
         .s (or .spstr): value as whitespace-separated string.
+        .p (or .paths): list of path objects
         
     Any values which require transformations are computed only once and
     cached.
@@ -871,6 +986,12 @@ class LSString(str):
     
     p = paths = property(get_paths)
 
+def print_lsstring(arg):
+    """ Prettier (non-repr-like) and more informative printer for LSString """
+    print "LSString (.p, .n, .l, .s available). Value:"
+    print arg
+    
+print_lsstring = result_display.when_type(LSString)(print_lsstring)
 
 #----------------------------------------------------------------------------
 class SList(list):
@@ -881,7 +1002,8 @@ class SList(list):
         .l (or .list) : value as list (the list itself).
         .n (or .nlstr): value as a string, joined on newlines.
         .s (or .spstr): value as a string, joined on spaces.
-
+        .p (or .paths): list of path objects
+        
     Any values which require transformations are computed only once and
     cached."""
 
@@ -917,6 +1039,72 @@ class SList(list):
     
     p = paths = property(get_paths)
 
+    def grep(self, pattern, prune = False):
+        """ Return all strings matching 'pattern' (a regex or callable) 
+        
+        This is case-insensitive. If prune is true, return all items
+        NOT matching the pattern.
+        
+        Examples::
+        
+            a.grep( lambda x: x.startswith('C') )
+            a.grep('Cha.*log', prune=1)
+        """
+        if isinstance(pattern, basestring):
+            pred = lambda x : re.search(pattern, x, re.IGNORECASE)
+        else:
+            pred = pattern
+        if not prune:
+            return SList([el for el in self if pred(el)])
+        else:
+            return SList([el for el in self if not pred(el)])
+    def fields(self, *fields):
+        """ Collect whitespace-separated fields from string list
+        
+        Allows quick awk-like usage of string lists.
+        
+        Example data (in var a, created by 'a = !ls -l')::
+            -rwxrwxrwx  1 ville None      18 Dec 14  2006 ChangeLog
+            drwxrwxrwx+ 6 ville None       0 Oct 24 18:05 IPython        
+        
+        a.fields(0) is ['-rwxrwxrwx', 'drwxrwxrwx+']
+        a.fields(1,0) is ['1 -rwxrwxrwx', '6 drwxrwxrwx+']
+        (note the joining by space).
+        
+        IndexErrors are ignored.
+        
+        Without args, fields() just split()'s the strings.
+        """
+        if len(fields) == 0:
+            return [el.split() for el in self]
+        
+        res = SList()
+        for el in [f.split() for f in self]:
+            lineparts = []
+
+            for fd in fields:
+                try:
+                    lineparts.append(el[fd])
+                except IndexError:
+                    pass
+            if lineparts:
+                res.append(" ".join(lineparts))
+            
+        return res        
+                
+            
+            
+        
+
+def print_slist(arg):
+    """ Prettier (non-repr-like) and more informative printer for SList """
+    print "SList (.p, .n, .l, .s, .grep(), .fields() available). Value:"
+    nlprint(arg)
+    
+print_slist = result_display.when_type(SList)(print_slist)
+
+
+
 #----------------------------------------------------------------------------
 def esc_quotes(strng):
     """Return the input string with single and double quotes escaped out"""
@@ -953,7 +1141,7 @@ def make_quoted_expr(s):
     else:
         # give up, backslash-escaped string will do
         return '"%s"' % esc_quotes(s)
-    res = itpl("$raw$quote$s$tailpadding$quote$tail")
+    res = raw + quote + s + tailpadding + quote + tail
     return res
 
 
@@ -1001,7 +1189,7 @@ def raw_input_ext(prompt='',  ps2='... '):
 
 #----------------------------------------------------------------------------
 def ask_yes_no(prompt,default=None):
-    """Asks a question and returns an integer 1/0 (y/n) answer.
+    """Asks a question and returns a boolean (y/n) answer.
 
     If default is given (one of 'y','n'), it is used if the user input is
     empty. Otherwise the question is repeated until an answer is given.
@@ -1227,7 +1415,8 @@ def get_pager_start(pager,start):
     return start_string
 
 #----------------------------------------------------------------------------
-if os.name == "nt":
+# (X)emacs on W32 doesn't like to be bypassed with msvcrt.getch()
+if os.name == 'nt' and os.environ.get('TERM','dumb') != 'emacs':
     import msvcrt
     def page_more():
         """ Smart pausing between pages
@@ -1686,6 +1875,70 @@ def map_method(method,object_list,*argseq,**kw):
         idx += 1
     return out_list
 
+#----------------------------------------------------------------------------
+def get_class_members(cls):
+    ret = dir(cls)
+    if hasattr(cls,'__bases__'):
+        for base in cls.__bases__:
+            ret.extend(get_class_members(base))
+    return ret
+
+#----------------------------------------------------------------------------
+def dir2(obj):
+    """dir2(obj) -> list of strings
+
+    Extended version of the Python builtin dir(), which does a few extra
+    checks, and supports common objects with unusual internals that confuse
+    dir(), such as Traits and PyCrust.
+
+    This version is guaranteed to return only a list of true strings, whereas
+    dir() returns anything that objects inject into themselves, even if they
+    are later not really valid for attribute access (many extension libraries
+    have such bugs).
+    """
+    
+    # Start building the attribute list via dir(), and then complete it
+    # with a few extra special-purpose calls.
+    words = dir(obj)
+
+    if hasattr(obj,'__class__'):
+        words.append('__class__')
+        words.extend(get_class_members(obj.__class__))
+    #if '__base__' in words: 1/0
+
+    # Some libraries (such as traits) may introduce duplicates, we want to
+    # track and clean this up if it happens
+    may_have_dupes = False
+
+    # this is the 'dir' function for objects with Enthought's traits
+    if hasattr(obj, 'trait_names'):
+        try:
+            words.extend(obj.trait_names())
+            may_have_dupes = True
+        except TypeError:
+            # This will happen if `obj` is a class and not an instance.
+            pass
+
+    # Support for PyCrust-style _getAttributeNames magic method.
+    if hasattr(obj, '_getAttributeNames'):
+        try:
+            words.extend(obj._getAttributeNames())
+            may_have_dupes = True
+        except TypeError:
+            # `obj` is a class and not an instance.  Ignore
+            # this error.
+            pass
+
+    if may_have_dupes:
+        # eliminate possible duplicates, as some traits may also
+        # appear as normal attributes in the dir() call.
+        words = list(set(words))
+        words.sort()
+
+    # filter out non-string attributes which may be stuffed by dir() calls
+    # and poor coding in third-party modules
+    return [w for w in words if isinstance(w, basestring)]
+
 #----------------------------------------------------------------------------
 def import_fail_info(mod_name,fns=None):
     """Inform load failure for a module."""
